{% if service_name is defined %}
# Environment files for {{ service_name }}
{% else %}
# Environment files for freva service
{% endif %}

{% if env_vars is defined and env_vars %}
{% for key, value in env_vars | dictsort %}
{{ key }}={{ value }}
{% endfor %}
{% else %}
# No environment variables defined
{% endif %}
