version: '3'
services:
  {{vault_name}}:
    user: root:0
    image: ghcr.io/freva-org/freva-vault:{{vault_version}}
    hostname: {{vault_name}}
    cap_add:
      - IPC_LOCK
      - SYS_NICE
    dns:
      - 8.8.8.8
      - 8.8.4.4
    environment:
      - ROOT_PW={{ root_passwd }}
      - KEY_FILE=/vault/file/keys
    volumes:
        - data:/vault/file:z
    container_name: {{vault_name}}
    tty: true
    ports:
      - 5002:5002

volumes:
  data:
