# Environment files for {{ service_name | default('freva service')}}

{% if env_vars is defined and env_vars %}
{% for key, value in env_vars | dictsort %}
{{ key }}={{ value }}
{% endfor %}
{% endif %}
{% if extra_env_vars is defined and extra_env_vars %}
{% for key, value in extra_env_vars | dictsort %}
{{ key }}={{ value }}
{% endfor %}
{% endif %}
